
From: john stultz <johnstul@us.ibm.com>

I've been hunting down a bug affecting IBM x440/x445 systems where the
floppy driver would get spurious interrupts and would not initialize
properly.  

After digging James Cleverdon and Bill Irwin pointed out that target_cpus()
is routing the interrupts to the clustered apic broadcast mask using
destFixed deliver mode.  This was causing multiple interrupts to show up,
breaking the floppy init code.  

This fix simply changes the delivery mode to dest_LowestPrio and initially
routes interrupts to the first cpu to resolve this issue.

Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/include/asm-i386/mach-summit/mach_apic.h |    8 ++++++--
 1 files changed, 6 insertions(+), 2 deletions(-)

diff -puN include/asm-i386/mach-summit/mach_apic.h~fix-target_cpus-for-summit-subarch include/asm-i386/mach-summit/mach_apic.h
--- 25/include/asm-i386/mach-summit/mach_apic.h~fix-target_cpus-for-summit-subarch	2004-09-02 18:09:02.424196400 -0700
+++ 25-akpm/include/asm-i386/mach-summit/mach_apic.h	2004-09-02 18:09:02.427195944 -0700
@@ -19,11 +19,15 @@
 
 static inline cpumask_t target_cpus(void)
 {
-	return CPU_MASK_ALL;
+	/* CPU_MASK_ALL (0xff) has undefined behaviour with
+	 * dest_LowestPrio mode logical clustered apic interrupt routing
+	 * Just start on cpu 0.  IRQ balancing will spread load
+	 */
+	return cpumask_of_cpu(0);
 } 
 #define TARGET_CPUS	(target_cpus())
 
-#define INT_DELIVERY_MODE (dest_Fixed)
+#define INT_DELIVERY_MODE (dest_LowestPrio)
 #define INT_DEST_MODE 1     /* logical delivery broadcast to all procs */
 
 static inline unsigned long check_apicid_used(physid_mask_t bitmap, int apicid)
_
